'use strict';

//region Imports
var path = require('path');
var fs = require('fs');
var _ = require('lodash');
//endregion

function getConfigPath() {
  return path.join(process.cwd(), './babelator.js');
}

function getConfig() {
  // If `babelator.js` exists, grab config settings from there.
  var babelatorConfigPath = getConfigPath();
  return fs.existsSync(babelatorConfigPath) ? require(babelatorConfigPath) : {};
}

function main() {

  var opts = _.defaultsDeep({}, getConfig(), {
    srcDir: 'src',
    destDir: 'lib',
    shouldCompileTests: true,
    //eslintrc: {}
    //mocha: {},
    babel: {}
  });

  // Computed config.
  _.defaultsDeep(opts, {
    srcGlob: opts.srcDir + '/**/*.js',
    destGlob: opts.destDir + '/**/*',
    testsGlob: opts.srcDir + '/**/__tests__/**/*-test.js',
    testsDestGlob: opts.destDir + '/**/__tests__/**/*-test.js' });

  ////////
  // This is shared.
  // Transpiled tests.
  var transformAsyncToModuleMethod = require('babel-plugin-transform-async-to-module-method');
  var asyncPlugin = [transformAsyncToModuleMethod, {
    module: 'bluebird',
    method: 'coroutine'
  }];
  var addModuleExports = require('babel-plugin-add-module-exports');
  var transformRuntime = require('babel-plugin-transform-runtime');
  var externalHelpers = require('babel-plugin-external-helpers-2');
  var babelDefaults = {
    plugins: [externalHelpers, asyncPlugin, transformRuntime, addModuleExports],
    presets: ['es2015', 'react', 'stage-0']
  };
  ////////

  _.defaultsDeep(opts.babel, babelDefaults);

  return opts;
}

main.getConfigPath = getConfigPath;

module.exports = main;