process.on('unhandledRejection', e => {throw e})

// TODO
// - Use init-package-json.
// - Use promises.

const main = async function() {

  //region Imports
  const fs = require('fs')
  const path = require('path')
  const cwd = require('cwd')
  const modifyPackageJson = require('./util/modifyPackageJson')
  const replaceFile = require('./util/replaceFile')
  const {spawnSync} = require('child_process')
  const init = require('init-package-json')
  const _ = require('lodash')
  const inquirer = require('inquirer')
  const {promisify} = require('bluebird')
  const Promise = require('bluebird')
  //endregion

  // npm init

  if (!fs.existsSync(cwd('package.json'))) {
    console.log('Initializing npm package...')
    spawnSync('npm init -f')
    //init(cwd(), 'package.json', {}, (e, data) => {
    //})
  }

  // gulp

  console.log('Adding gulp to package.json')

  await promisify(modifyPackageJson)(json => {
    _.set(json, 'scripts.gulp', './node_modules/gulp/bin/gulp.js')
    return json
  })

  // gulpfile.js

  console.log('Updating gulpfile...')

  var gulpfileContents =
    "var gulp = require('gulp')\n" +
    "require('babelator')(gulp)\n"

  await promisify(replaceFile)(cwd('gulpfile.js'), gulpfileContents)

  // Add babelator dependency

  console.log('Adding babelator dependency...')

  const confirm = (message) => {
    return new Promise((resolve, reject) => {
      inquirer.prompt([{type: 'confirm', name: 'confirm', message}], (answers) => {
        resolve(answers)
      })
    })
  }

  async function installDependencies() {

    // TODO(vjpr): Provide options instead of 2 confirms. Duh!
    let answers = null

    answers = await confirm('Run `npm i -D babelator gulpjs/gulp.git#4.0`?')
    if (answers.confirm) {
      spawnSync('npm', ['i', '-D', 'babelator', 'gulpjs/gulp.git#4.0'], {stdio: 'inherit'})
      return
    }

    // Allows us to not have to wait for slow `npm install`.
    answers = await confirm('Run `npm link -D babelator gulp`?')
    if (answers.confirm) {
      spawnSync('npm', ['link', 'babelator', 'gulp'], {stdio: 'inherit'})
      return
    }

    // Instructions
    console.log('Please run `npm install`')
    console.log('Tip: Use `npm_lazy` module to locally cache npm modules.')

  }

  // TODO(vjpr): We should remove react from here and make it optional in another bootstrap task.
  // But it is currently used somewhere in babelator.
  async function installBabelDevDependencies() {

    let answers = null

    answers = await confirm('Run `npm link -D babel-preset-es2015 babel-preset-stage-0 babel-preset-react`?')
    if (answers.confirm) {
      spawnSync('npm', ['link', 'babel-preset-es2015', 'babel-preset-stage-0', 'babel-preset-react'], {stdio: 'inherit'})
      return
    }

    answers = await confirm('Run `npm i -D babel-preset-es2015 babel-preset-stage-0 babel-preset-react')
    if (answers.confirm) {
      spawnSync('npm', ['i', 'babel-preset-es2015', 'babel-preset-stage-0', 'babel-preset-react'], {stdio: 'inherit'})
      return
    }

  }

  async function installBabelRuntimeDependencies() {

    // For regenerator when using async.
    let deps = `babel-runtime bluebird`

    let answers = null

    answers = await confirm(`Run \`npm link -S ${deps}\`?`)
    if (answers.confirm) {
      spawnSync('npm', ['link', 'babel-runtime', 'bluebird'], {stdio: 'inherit'})
      return
    }

    answers = await confirm(`Run \`npm i -S ${deps}\`?`)
    if (answers.confirm) {
      spawnSync('npm', ['i', 'babel-runtime@^5', 'bluebird'], {stdio: 'inherit'})
      return
    }

  }

  await installDependencies()
  await installBabelDevDependencies()
  await installBabelRuntimeDependencies()

  console.log('Done!')

}

function cli() {
  main().then(() => {})
}

module.exports = cli
